#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === CaseParser
#
# This class will scan a file and extract all case entries.
#
# For now though, we will work only on the first case menu.
# =========================================================================== #
# require 'case_parser.rb'
# =========================================================================== #
require 'case_parser/version/version.rb'

class CaseParser # CaseParser.new

  begin
    require 'opn'
  rescue LoadError; end

  begin
    require 'convert_global_env'
  rescue LoadError; end

  begin
    require 'colours'
    include Colours
  rescue LoadError; end

  # ========================================================================= #
  # === USE_THIS_ENCODING
  # ========================================================================= #
  USE_THIS_ENCODING = 'UTF-8' # 'ISO-8859-1'

  # ========================================================================= #
  # Test it via compile.rb.
  # ========================================================================= #
  DEFAULT_FILE =
    "#{ENV['RBT']}/compile.rb"
  # DEFAULT_FILE = '/Depot/jjj/testcase.rb'

  # ========================================================================== #
  # === SHALL_WE_DEBUG
  # ========================================================================== #
  SHALL_WE_DEBUG = false

  # ========================================================================== #
  # === initialize
  # ========================================================================== #
  def initialize(
      i           = DEFAULT_FILE,
      run_already = true
    )
    reset
    case run_already
    when :continue,
         :do_not_exit
      @may_we_exit_on_missing_file = false
      run_already = true
    end
    run(i) if run_already
  end

  # ========================================================================== #
  # === set_file
  #
  # Simply set the @file variable here.
  # ========================================================================== #
  def set_file(i)
    i = DEFAULT_FILE if i.nil?
    case i
    # ======================================================================== #
    # === :beautiful_menu
    # ======================================================================== #
    when :beautiful_menu
      i = ConvertGlobalEnv[
        '$RUBY_SRC/beautiful_url/lib/beautiful_url/toplevel_methods/menu.rb'
      ] # Hardcoded.
      unless File.exist? i
        opn; e 'Warning - no file exists at `'+sfile(i)+'`.'
      end
    when :dia_menu
      i = ConvertGlobalEnv['$RUBY_SRC/diamond_shell/lib/diamond_shell/menu/menu.rb']
      unless File.exist? i
        opn; e 'Warning - no file exists at `'+sfile(i)+'`.'
      end
    end
    i = i.to_s unless i.is_a? String
    i = ConvertGlobalEnv.convert(i) if i.include? '$'
    @file = i
  end

  # ========================================================================== #
  # === reset  
  # ========================================================================== #
  def reset
    # ======================================================================== #
    # === @dataset
    # ======================================================================== #
    @dataset = nil
    # ======================================================================== #
    # === @seen_case
    #
    # Whether we saw "case" yet or did not.
    # ======================================================================== #
    @seen_case = false
    # ======================================================================== #
    # === @array_keeping_all_when_entries
    # ======================================================================== #
    @array_keeping_all_when_entries = []
    # ======================================================================== #
    # === @shall_we_debug
    #
    # If true then we will debug.
    # ======================================================================== #
    @shall_we_debug = SHALL_WE_DEBUG
    # ======================================================================== #
    # === @may_we_exit_on_missing_file
    # ======================================================================== #
    @may_we_exit_on_missing_file = true
  end

  # ========================================================================== #
  # === feedback
  # ========================================================================== #
  def feedback
    opn; pp results
  end

  # ========================================================================== #
  # === sanitize_array
  #
  # This will try to sanitize the array.
  # ========================================================================== #
  def sanitize_array
    @array_keeping_all_when_entries.flatten!
    begin # Musct rescue this in case the encoding is bad.
      @array_keeping_all_when_entries.map! {|entry|
        entry.delete("'").strip
      }
    rescue; end
    # ======================================================================== #
    # Now, we remove all Regexes from this menu, because Regexes
    # are not really useful to keep for tab completion in most
    # scripts.
    # ======================================================================== #
    @array_keeping_all_when_entries.reject! {|entry|
      entry.start_with? '/' or
      entry.start_with? ':' # Symbols also get rejected.
    }
  end

  # ========================================================================== #
  # === remove_potential_comments
  #
  # This gets rid of potential '#' comments.
  # ========================================================================== #
  def remove_potential_comments(i)
    i = i[0, i.index('#')].strip if i.include? '#' # If there is a #, return all up to that.
    return i
  end

  # ========================================================================== #
  # === red
  # ========================================================================== #
  def red
    Colours::RED
  end

  # ========================================================================== #
  # === read_file
  # ========================================================================== #
  def read_file
    if File.exist? @file
      @dataset = File.readlines(@file, encoding: USE_THIS_ENCODING)
    else
      if @may_we_exit_on_missing_file
        opn; e red+'Can not continue. File `'+sfile(@file)+
               red+'` does not exist.'+rev
        exit
      end
    end
    if @shall_we_debug
      opn; pp dataset?
    end
  end

  # ========================================================================== #
  # === report_n_entries
  # ========================================================================== #
  def report_n_entries
    opn; e "We found #{simp(dataset?.size.to_s)} entries."
  end

  # ========================================================================== #
  # === results
  # ========================================================================== #
  def results
    _ = @array_keeping_all_when_entries
    _.sort! if _
    _
  end

  # ========================================================================== #
  # === extract_first_when
  #
  # Use this method here if you wish to extract the first word.
  # ========================================================================== #
  def extract_first_when(
      mode = :extract_only_first_entry
    )
    dataset?.each { |d|
      if d.include? 'when' # Work on when-entries only. This is not perfect, as we miss other entries.
        begin
          _ = d.chomp.strip.gsub(/when /,'') # Eliminate 'when ' here.
        rescue ArgumentError
          opn; e 'ArgumentError - invalid byte sequence in US-ASCII'
        end
        if _
          _ = _.split(',') if _.include? ',' # Split on ',' if they exist.
          _ = _.delete("'") if _.include? "'" # Eliminate "'" characters.
          append_to_main_array(_)
        end
      end
    }
  end

  # ========================================================================== #
  # === extract_case_menu
  #
  # This method will extract the case menu.
  # ========================================================================== #
  def extract_case_menu
    _ = []
    dataset?.each {|line|
      begin
        @seen_case = true if line.include? 'case'
        _ << remove_potential_comments(line) if @seen_case
        @seen_case = false if line =~ /end$/
      rescue ArgumentError # This is here to rescue against "invalid byte sequence in US-ASCII"
      end
    } if dataset?
    _.reject!(&:empty?)
    @dataset = _
  end

  # ========================================================================== #
  # === append_to_main_array
  # ========================================================================== #
  def append_to_main_array(i)
    @array_keeping_all_when_entries << i
  end

  # ========================================================================== #
  # === dataset?
  # ========================================================================== #
  def dataset?
    @dataset
  end

  # ========================================================================== #
  # === found_entries?
  #
  # We return all entries found in a sorted manner here.
  # ========================================================================== #
  def found_entries?
    @array_keeping_all_when_entries.sort
  end

  # ========================================================================== #
  # === run
  #
  # General run method. We will also set_file() in this method.
  # ========================================================================== #
  def run(which_file)
    set_file(which_file)
    read_file
    extract_case_menu
    extract_first_when
    sanitize_array
  end

end

if __FILE__ == $PROGRAM_NAME
  _ = CaseParser.new(ARGV.first)
  _.feedback
  _.report_n_entries
end # cparse
    # cparse $RUBY_SRC/beautiful_url/lib/beautiful_url/menu/beautiful_menu.rb
    # 